// Coordinator.java
// Written by: Maria Kazandjieva, marykaz@hotmail.com, Summer 2005
// Created: July/August 05

// This is the main class that is run on the ZigBee corrdinator machine.

// Its tasks include obtaining information from the RFID tags
// and sending it via Bluetooth to the Server computer


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.bluetooth.*;
import javax.swing.*;
import javax.comm.*;
import java.util.*;
import javax.microedition.io.*;

public class Coordinator implements ActionListener{

	//declare and initiate variables
	JButton  write = new JButton("Write and Read tags");
	JButton read = new JButton ("Read tags");
	JButton deviceB = new JButton("Find Device");
	JButton serviceB = new JButton("Find Service");
	JButton sendB = new JButton("Send data via Bluetooth to the Server");
	JLabel orL= new JLabel("--------------- OR -----------------");

	JTextArea textarea = new JTextArea("",20, 40);

	DeviceDiscoverer deviceDiscoverer = null;
	ServiceDiscoverer serviceDiscoverer = null;

	JFrame frame;

	public  Coordinator(){

	// create the visual components using Swing
		frame = new JFrame("Coordinator");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		JScrollPane scrollPane = new JScrollPane(textarea);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		textarea.setEditable(false);

		Container cp = frame.getContentPane();
		cp.setLayout(new BorderLayout());

		JPanel buttonsPane = new JPanel();
		buttonsPane.setLayout(new GridLayout(1,2));

		JPanel bottomPane = new JPanel();
		bottomPane.setLayout(new GridLayout(3,1));

		write.addActionListener(this);
		buttonsPane.add(write);
		write.setEnabled(true);

		buttonsPane.add(orL);

		read.addActionListener(this);
		buttonsPane.add(read);
		read.setEnabled(true);

		sendB.addActionListener(this);
		deviceB.addActionListener(this);
		serviceB.addActionListener(this);

		sendB.setEnabled(false);
		deviceB.setEnabled(false);
		serviceB.setEnabled(false);

		cp.add(buttonsPane, BorderLayout.NORTH);

		cp.add(scrollPane, BorderLayout.CENTER);

		bottomPane.add(deviceB);
		bottomPane.add(serviceB);
		bottomPane.add(sendB);

		cp.add(bottomPane, BorderLayout.SOUTH);

		updateStatus("Plase select one of the two options above");
		updateStatus ("Then click the appropriate button");
		updateStatus("");

		frame.pack();
		frame.setVisible(true);

	}

	public void actionPerformed(ActionEvent e) {

	// if the write button is clicked, the user has to write and read the 5 RFID tags
				if( e.getSource() == write) {
					startWrite();
					deviceB.setEnabled(true);
				}
	// if the read button is clicked, the user only has to read tag data
				if( e.getSource() == read) {
					startRead();
					deviceB.setEnabled(true);
					write.setEnabled(false);
					read.setEnabled(false);
				}
	// clicking the deviceB button calls the method that searches for remote devices
				if( e.getSource() == deviceB) {
					deviceDiscoverer = new DeviceDiscoverer(this);
					serviceB.setEnabled(true);
					deviceB.setEnabled(false);
				}
	//clicking the serviceB button calls the method that searches for a matching service on a device
				if( e.getSource() == serviceB) {
					serviceDiscoverer = new ServiceDiscoverer(this, deviceDiscoverer.remoteDevices);
					sendB.setEnabled(true);
					serviceB.setEnabled(false);
				}
	// the sendB button triggers the method that will send information over the Bluetooth connection
				if( e.getSource() == sendB) {
					RunRead runRFID = new RunRead(this);
					runRFID.sendData(serviceDiscoverer.connectionURL);
					sendB.setEnabled(false);
				}
		}

	// this method is used to update the messages that appear in the text area
	public void updateStatus(String message){
		textarea.append("\n" + message);
	}


	// this method creates and starts a new thread to write and read the tags
	// the threading is needed to that the GUI components can function without freezing
	public void startWrite(){

		Thread t = new Thread (new RunWrite(this));
		t.start();
	}
	// this method creates and starts a thread that reads the RFID tags
	public void startRead(){

		Thread t2 = new Thread (new RunRead(this));
		t2.start();
	}

	// main method for this class. this is where execution starts.
	public static void main(String[] args) {
			new Coordinator();

	} //end main

}
