// RunWrite.java
// Maria Kazandjieva, marykaz@hotmail.com, Summer 2005
// This class implements the Runnable interface in order to support threaded application.
// It is needed so that the program can run without the Swing components freezing on the screen
// When this thread is ran the user has to first write the tags and then read them

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import javax.comm.*;
import java.util.*;
import javax.bluetooth.*;
import javax.microedition.io.*;

public class RunWrite implements Runnable {

	// declare and initiate variables

	public static int index = 0;

	public static Coordinator v;

	final int MAX = 100;
	public static int numberNodes=5;
	public static double[] data = new double[numberNodes];
	public static double[] energy = new double[numberNodes];
	public static int[] x = new int[numberNodes];
	public static int[] y = new int[numberNodes];

	public static String[] rfdata = new String[numberNodes];

	// we pass the main class Coordinator so we can access its updateStatus method
	public RunWrite (Coordinator v){
		this.v = v;
	}

//this is where the thread will start executing when it is started
	public void run() {
		try {
			start();

		}
		catch (IOException e) {e.getMessage();}
		catch (NoSuchPortException e){e.getMessage();}
		catch (PortInUseException e){e.getMessage();}
		catch (UnsupportedCommOperationException e) {e.getMessage();}
		catch (InterruptedException e) {e.getMessage();}

	}


	//this is the method, it starts by finding COM1, then calls a method to open COM1
		public static void start() throws IOException, NoSuchPortException, PortInUseException,
				UnsupportedCommOperationException, InterruptedException {


			CommPortIdentifier portID;
			Enumeration pList;
			pList = CommPortIdentifier.getPortIdentifiers();

			while (pList.hasMoreElements()) {
				portID = (CommPortIdentifier)pList.nextElement();

				if (portID.getPortType() == CommPortIdentifier.PORT_SERIAL) {

					if (portID.getName().equals("COM1")) {

						OpenCommPort(portID);
					}
				}
			}
		}

		//this method opens COM1
		public static void OpenCommPort(CommPortIdentifier portID) throws IOException, NoSuchPortException, PortInUseException,
				UnsupportedCommOperationException, InterruptedException {
			//This section of the code attempts to open port COM1

			 CommPort thePort;
			thePort = portID.open("SerailApp", 3000);

			SerialPort myPort;
			myPort = (SerialPort)thePort;

			//set up serial port parameters
			myPort.setSerialPortParams(9600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);

			String TagID = "";


			decision(myPort, thePort, TagID);
	}


	// this method decides whether to call the writeTag or readTag method
	public static void decision(SerialPort myPort, CommPort thePort, String TagID)
						throws IOException, NoSuchPortException, PortInUseException,
						UnsupportedCommOperationException, InterruptedException {

				if (index <numberNodes) {
					writeTag(myPort, thePort, TagID);
				}
				if (index>=numberNodes && index<numberNodes*2) {
					readTag(myPort, thePort, TagID);
			}
		}


	// this method drives the writeTag command
		public static void writeTag(SerialPort myPort, CommPort thePort, String TagID) throws IOException, NoSuchPortException, PortInUseException,
				UnsupportedCommOperationException, InterruptedException {

			Requests req = new Requests();
			readResponse read = new readResponse();
			responseHandler respHand = new responseHandler();

			// try writing a tag.
			// the integer value returned indicates whether there was a tag or not
			int success = req.writeTag(myPort, thePort);

			if (success==0) {
				v.updateStatus("There was no tag detected. Try again.");
				v.updateStatus("");
				// if the selection of a tag was not successful program sleeps and tries again
				Thread.sleep(5000);
				decision(myPort, thePort, TagID);
			}
			else {
				// if there was a tag, depending on the responce from the reader
				// we do different things
				StringBuffer stringBuff = read.readResponse (myPort, thePort);
				String respCode = respHand.getResponseCode(stringBuff);

			// code 44 means the writeTag was successful
			if(respCode.equals("44")){

					v.updateStatus("[" + (index+1) + "]  The writeTag was successful");
					v.updateStatus("");
					index++;
					Thread.sleep(5000);
					// get ready to write or read the next tag
					decision(myPort, thePort, TagID);

			}
			else {
				//System.out.println("Response Code: " + respCode);
				//System.out.println("The writeTag FAILED!");
				v.updateStatus("Response Code:" + respCode);
				v.updateStatus("The writeTag FAILED!");
				v.updateStatus("");
				Thread.sleep(5000);
				decision(myPort, thePort, TagID);
			}
		}


		} // end writeTag method

		// this method drives the readTag command
		public static void readTag(SerialPort myPort, CommPort thePort, String TagID) throws IOException, NoSuchPortException, PortInUseException,
					UnsupportedCommOperationException, InterruptedException {

		Requests req = new Requests();
		readResponse read = new readResponse();
		responseHandler respHand = new responseHandler();

		int success = req.readTag(myPort, thePort);

		if (success ==0) {
			v.updateStatus("There was no tag detected. Try again.");
			v.updateStatus("");
			Thread.sleep(5000);
			decision(myPort, thePort, TagID);
		}
		else {

			StringBuffer stringBuff = read.readResponse (myPort, thePort);
			String respCode = respHand.getResponseCode(stringBuff);
			String tagData = respHand.getTagData(stringBuff);

			// code 24 means the reading of the tag was successful
			if(respCode.equals("24")){

				v.updateStatus("[" + (index+1-5) + "]  The readTag was successful");
				v.updateStatus("Data read is " + tagData);
				v.updateStatus("");

				// store the data that has been read from the tag in a string array
				rfdata[index-5]=tagData;

				index++;
				Thread.sleep(5000);
				decision(myPort, thePort, TagID);
			}
			else {

				v.updateStatus("The readTag FAILED");
				v.updateStatus("");
				Thread.sleep(5000);
				decision(myPort, thePort, TagID);
			}
		}

	} // end readTag method


// this method parses the data from the tags and sends it to the remote Bluetooth server
	public void sendData(String connectionURL){

	// parsing and converting the data and storing it in appropriate arrays
	for (int i=0; i<numberNodes; i++) {
	data[i]=(double)Integer.parseInt(rfdata[i].substring(0,2));
	energy[i]=(double)Integer.parseInt(rfdata[i].substring(2,4));
	x[i]=Integer.parseInt(rfdata[i].substring(4,6));
	y[i]=Integer.parseInt(rfdata[i].substring(6,8));
	}
	try{
		// connect to the remote bluetooth server
		StreamConnection connection = (StreamConnection)Connector.open(connectionURL);

		// open an output stream to send the data
		DataOutputStream out = connection.openDataOutputStream();

		v.updateStatus("[coordinator:] The output stream is now open to send data");

		out.writeInt(numberNodes);

		v.updateStatus("[coordinator:] numberNodes = " + numberNodes);

		// send data using the data output stream
		// display messages in the swing text aread to confirm data was sent
		for (int i=0; i<numberNodes; i++) {
			out.writeDouble(data[i]);
		}
		v.updateStatus("[coordinator:] Data values were sent");

		for (int i=0; i<numberNodes; i++) {
			out.writeDouble(energy[i]);
		}
		v.updateStatus("[coordinator:] Energy values were sent");

		for (int i=0; i<numberNodes; i++) {
			out.writeInt(x[i]);
		}
		v.updateStatus("[coordinator:] X-coordinates were sent");

		for (int i=0; i<numberNodes; i++) {
			out.writeInt(y[i]);
		}
		v.updateStatus("[coordinator:] Y-coordinates were sent");

		v.updateStatus("[coordinator:] Sending information completed successfully!");


		try{
			Thread.sleep(2000);
		} catch (Exception e){
			}

	// close the stream and connection after being done
	out.close();
	connection.close();

	}catch(Exception e){
		e.getMessage();
	}

	} // end of method

} // end of class