// ServiceDiscoverer.java
// Modified by: Maria Kazandjieva, marykaz@hotmail.com, Summer 2005

// After remote devices have been found, the program has to search for services that match the needed one.

import javax.bluetooth.*;
import java.io.*;
import java.util.Vector;


public class ServiceDiscoverer implements DiscoveryListener {

	UUID[] uuidSet = {new UUID("8821", true)};
	int[] attrSet = {0x0100, 0x0003, 0x0004};

	Coordinator zigbee;
	ServiceRecord serviceRecord;
	String connectionURL;


	public ServiceDiscoverer(Coordinator zigbee, Vector deviceList) {

		this.zigbee = zigbee;

		try {
			LocalDevice localDevice = LocalDevice.getLocalDevice();
			DiscoveryAgent discoveryAgent = localDevice.getDiscoveryAgent();
			RemoteDevice remoteDevice = null;

			for(int i=0; i<deviceList.size(); i++){

				remoteDevice = (RemoteDevice)deviceList.get(i);

				zigbee.updateStatus("[coordinator:] Searching for Services on: "  + remoteDevice.getFriendlyName(true)+ " (" + remoteDevice.getBluetoothAddress() + ")" );
				discoveryAgent.searchServices(attrSet, uuidSet, remoteDevice, this);
				try{
					Thread.sleep(2000);
				} catch (Exception e){
				}

			}

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}


	public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {

		for(int i = 0; i < servRecord.length; i++) {

	        DataElement serviceNameElement = servRecord[i].getAttributeValue(0x0100);
			String serviceName = (String)serviceNameElement.getValue();
			zigbee.updateStatus("[coordinator:] A new service is discovered: " + serviceName );

			if(serviceName.equals("DataTransfer")){

				zigbee.updateStatus("[coordinator:] A matching service has been found");

				try {
					connectionURL = servRecord[i].getConnectionURL(1,false);
				} catch (Exception e){
					zigbee.updateStatus("[coordinator:] oops");
				}

				zigbee.updateStatus("[coordinator:] The connection URL is: " + connectionURL );

			}
		}
	}



	public void serviceSearchCompleted(int transID, int respCode) {

		String searchStatus = null;

		if (respCode == DiscoveryListener.SERVICE_SEARCH_DEVICE_NOT_REACHABLE) {
			searchStatus = "Device not reachable";
		}
		else if (respCode == DiscoveryListener.SERVICE_SEARCH_NO_RECORDS) {
			searchStatus = "Service not available";
		}
		else if (respCode == DiscoveryListener.SERVICE_SEARCH_COMPLETED) {
			searchStatus = "Service search completed";
		}
		else if (respCode == DiscoveryListener.SERVICE_SEARCH_TERMINATED) {
			searchStatus = "Service search terminated";
		}
		else if (respCode == DiscoveryListener.SERVICE_SEARCH_ERROR) {
			searchStatus = "Service search error";
		}

		zigbee.updateStatus("[corrdinator:] " + searchStatus);
	}


    public void inquiryCompleted(int discType){}

    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod){}

}